@@ -1,5 +1,5 @@
 /* Test suite for version-etc.
-   Copyright (C) 2009-2016 Free Software Foundation, Inc.
+   Copyright (C) 2009-2017 Free Software Foundation, Inc.
    This file is part of the GNUlib Library.
 
    This program is free software: you can redistribute it and/or modify
@@ -19,14 +19,12 @@
 
 #include "version-etc.h"
 
-#include "progname.h"
 
 #define AUTHORS "Sergey Poznyakoff", "Eric Blake"
 
 int
 main (int argc _GL_UNUSED, char **argv)
 {
-  set_program_name (argv[0]);
   version_etc (stdout, "test-version-etc", "dummy", "0", AUTHORS,
                (const char *) NULL);
   return 0;
