@@ -23,7 +23,6 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include "binary-io.h"
 #include "configmake.h"
 #include "dirname.h"
 #include "ignore-value.h"
@@ -108,4 +107,26 @@ static _GL_UNUSED void
 __asan_unpoison_memory_region (void const volatile *addr, size_t size) { }
 #endif
 
+#ifndef FALLTHROUGH
+# if __GNUC__ < 7
+#  define FALLTHROUGH ((void) 0)
+# else
+#  define FALLTHROUGH __attribute__ ((__fallthrough__))
+# endif
+#endif
+
+/* When we deliberately use duplicate branches, use this macro to
+   disable gcc's -Wduplicated-branches in the containing expression.  */
+#if 7 <= __GNUC__
+# define IGNORE_DUPLICATE_BRANCH_WARNING(exp)				\
+  ({									\
+    _Pragma ("GCC diagnostic push")					\
+    _Pragma ("GCC diagnostic ignored \"-Wduplicated-branches\"")	\
+    exp;								\
+    _Pragma ("GCC diagnostic pop")					\
+  })
+#else
+# define IGNORE_DUPLICATE_BRANCH_WARNING(exp) exp
+#endif
+
 #endif
